#include "Iso.h"
#include "Shader.h"
#include "AllDef.h"
#include "Draw.h"
#include <windows.h>
#include "vec.h"
#include "str.h"

extern int stereo_on;

IsoViewer::IsoViewer():l1(0),b1(0),b2(1),st(0),st_i(0),st_n(0),ps(0)
{

	draw_frame_is=1;
	
	ps = new ShaderProgram("srf.vs","srf.fs");	//      GPU +  


	// uniform-  
	SetAnag(0,0);
	SetBoundingBox(b1,b2);
	SetLevel(l1);

}



// -  -
void IsoViewer::SetLevel(float nl1)
{
	l1=clamp(0.0f,1.0f,nl1);

	ps->Use();
	ps->SetVar("level1", nl1);
	ps->UnUse();
}

//    
void IsoViewer::SetBoundingBox(vec3 a,vec3 b)
{
	
	b1=vec3::Clamp(0,1,a);
	b2=vec3::Clamp(0,1,b);

	ps->Use();
	ps->SetVar("box1",b1);
	ps->SetVar("box2",b2);
	ps->SetVar("step_length",b1.length(b2)*0.002f);
	ps->UnUse();
}

//    
void IsoViewer::ApplyCamera(Camera* c)
{
	ps->Use();
	ps->SetVar("pos",c->GetStereoPosition());
	ps->SetVar("nav",c->GetNav());
	ps->UnUse();
}

void IsoViewer::Draw(Camera* c)
{
	vec3 bb1=b1,bb2=b2;

	// 
	if(stereo_on!=1 && draw_frame_is) 
	{
		glColor3f(0.5f,0.5f,0.5f);
		glDisable(GL_LIGHTING);
		DrawCubeFrame(bb1,bb2);
		glEnable(GL_LIGHTING);
		
	}

	ps->Use();
	ps->SetVar("box1",b1);
	ps->SetVar("box2",b2);

	DrawCube(bb1,bb2); //  (..   , ..       )
	ps->UnUse();



}
IsoViewer::~IsoViewer()
{
	delete ps;
	if(st)delete st;
	if(st_i)delete st_i;
	if(st_n)delete st_n;
}

/*
#define CELL_SIZE 16
//    (data- , size- , c- , size_b- )
template<class T>
void MakeBoundingCells(T*data,ivec3 size,T*c,ivec3 size_b)
{
	int id;
	//int sz = (size.x*size.y*size.z)/(CELL_SIZE*CELL_SIZE*CELL_SIZE);
	
//	memset(c,0,sz*sizeof(float));
	

	for(int i=0;i<size_b.x;i++)
	for(int j=0;j<size_b.y;j++)
	for(int k=0;k<size_b.z;k++)
	{
		bool fl=true;
		T max=-9999;
		T min=9999;
		for(int ii=i*size.x/(float)size_b.x-1;ii<(i+1)*size.x/(float)size_b.x+1;ii++)
		for(int jj=j*size.y/(float)size_b.y-1;jj<(j+1)*size.y/(float)size_b.y+1;jj++)
		for(int kk=k*size.z/(float)size_b.z-1;kk<(k+1)*size.z/(float)size_b.z+1;kk++)
			if( ivec3(ii,jj,kk).InBounds(ivec3(0),size) )
			{
				T vl = data[(ii + size.x*(jj + size.y*kk))];
				if(max<vl || fl)max=vl;
				if(min>vl || fl)min=vl;
				fl=false;
			}
		id=2*(i+(size_b.x)*(j+k*(size_b.y)));
		c[id] = min;
		c[id+1] = max;
		
	}
}

//  
void MakeNormals(BYTE*data,ivec3 size,float*n)
{
	for(int i=0;i<size.x;i++)
	for(int j=0;j<size.y;j++)
	for(int k=0;k<size.z;k++)
	{
		int id = i+size.x*(j+size.y*k);
		int vl = (int)data[id];
		if(i && i!=size.x-1)
			n[id*3]=(int)data[i+1+size.x*(j+size.y*k)]-(int)data[i-1+size.x*(j+size.y*k)];
		else
		{
		if(!i)
			n[id*3]=(int)data[i+1+size.x*(j+size.y*k)]-vl;
		else
			n[id*3]=vl-(int)data[i-1+size.x*(j+size.y*k)];
		}

		if(j && j!=size.y-1)
			n[id*3+1]=(int)data[i+size.x*(j+1+size.y*k)]-(int)data[i+size.x*(j-1+size.y*k)];
		else
		{
		if(!j)
			n[id*3+1]=(int)data[i+size.x*(j+1+size.y*k)]-vl;
		else
			n[id*3+1]=vl-(int)data[i+size.x*(j-1+size.y*k)];
		}

		if(k && k!=size.z-1)
			n[id*3+2]=(int)data[i+size.x*(j+size.y*(k+1))]-(int)data[i+size.x*(j+size.y*(k-1))];
		else
		{
		if(!k)
			n[id*3+2]=(int)data[i+size.x*(j+size.y*(k+1))]-vl;
		else
			n[id*3+2]=vl-(int)data[i+size.x*(j+size.y*(k-1))];
		}
	}
}
*/

//     GPU
void IsoViewer::UploadFieldData(void*data,ivec3 size, int type)
{
	if(st)delete st;


	//    3  +    
	st = new SimText3D(size.x,size.y,size.z,1,data,1,true,type);

/*
	ivec3 isz = size/CELL_SIZE;
	
	isz.x += !!(size.x%CELL_SIZE);
	isz.y += !!(size.y%CELL_SIZE);
	isz.z += !!(size.z%CELL_SIZE);

	int sz = 2*isz.x*isz.y*isz.z;
	void *dt2;
	dt2 = new BYTE[sz];

	MakeBoundingCells<BYTE>((BYTE*)data,size,(BYTE*)dt2,isz);

	if(st_i)delete st_i;
	st_i = new SimText3D(isz.x,isz.y,isz.z,2,dt2,2,false,GL_UNSIGNED_BYTE);
	delete[]dt2;

*/
	ps->Use();
	ps->SetVar("f_text",st->GetTexture());
//	ps->SetVar("f_text_i",st_i->GetTexture());
	ps->UnUse();


}

//   s  ( left)   
void IsoViewer::SetAnag(int s, bool left)
{
	
	switch(s)
	{
	case 0: // no stereo
		anag[0].set(1,0,0);
		anag[1].set(0,1,0);
		anag[2].set(0,0,1);
		break;
	case 1: // True anaglyh
		if(left)
		{
			anag[0].set(0.299,0.587,0.114);
			anag[1].set(0,0,0);
			anag[2].set(0,0,0);
		}else
		{
			anag[0].set(0,0,0);
			anag[1].set(0,0,0);
			anag[2].set(0.299,0.587,0.114);
		}
		break;
	case 2: // Gray anaglyh
		if(left)
		{
			anag[0].set(0.299,0.587,0.114);
			anag[1].set(0,0,0);
			anag[2].set(0,0,0);
		}else
		{
			anag[0].set(0,0,0);
			anag[1].set(0.299,0.587,0.114);
			anag[2].set(0.299,0.587,0.114);
		}
		break;
	case 3: // Color anaglyh
		if(left)
		{
			anag[0].set(1,0,0);
			anag[1].set(0,0,0);
			anag[2].set(0,0,0);
		}else
		{
			anag[0].set(0,0,0);
			anag[1].set(0,1,0);
			anag[2].set(0,0,1);
		}
		break;	
	case 4: // Half-Color anaglyh
		if(left)
		{
			anag[0].set(0.299,0.587,0.114);
			anag[1].set(0,0,0);
			anag[2].set(0,0,0);
		}else
		{
			anag[0].set(0,0,0);
			anag[1].set(0,1,0);
			anag[2].set(0,0,1);
		}
		break;	
	case 5: // Optimized anaglyh
		if(left)
		{
			anag[0].set(0,0.7,0.3);
			anag[1].set(0,0,0);
			anag[2].set(0,0,0);
		}else
		{
			anag[0].set(0,0,0);
			anag[1].set(0,1,0);
			anag[2].set(0,0,1);
		}
		break;	
	};

	ps->Use();
	ps->SetVar("RFrom",anag[0]);
	ps->SetVar("GFrom",anag[1]);
	ps->SetVar("BFrom",anag[2]);
	ps->UnUse();
}
