#include "AllInc.h"

#include <GL/glfw.h>
#include <stdio.h>
#include <stdlib.h>
#include "AllDef.h"
#include "Mouse.h"
#include "Camera.h"
#include "Draw.h"
#include "Iso.h"
#include "output.h"
#include <Math.h>
#include <ivec3.h>
#include "Draw2D.h"



Mouse mouse;
Camera cam;

IsoViewer*iso;

int keyboard[255];
int width = 450, height = 450;

int stereo_on=0, anag=1;


void SetStereoMode(int);
void InitInterlace();
void FullDraw();

//        x,y
vec3 GetNav(int x,int y)
{
	vec3 ll=cam.GetLeft()*((x-width/2)/float(height));
	vec3 uu=cam.GetTop()*((y-height/2)/float(height));
	
	
	return cam.GetNav()-(uu-ll)*(2*TAN_22_5);

}

//    dot      x,y
vec3 MoveDot(int x,int y,vec3 dot)
{
	vec3 n = cam.GetNav();
	float mx = max(abs(n.x),max(abs(n.y),abs(n.z)));
	flat f(dot-cam.GetPosition(),vec3(mx==abs(n.x),mx==abs(n.y),mx==abs(n.z)));
	return vec3::flat_cross_line(f,GetNav(x,y))+cam.GetPosition();
}

//  
void MouseMove ( int x, int y )
{
	mouse.Move(x,y);
	cam.MouseMove(x,y);

	if(keyboard['1']) 
	{
		iso->SetLevel(iso->GetMinLevel()+mouse.dy*0.001f);
		printf("%f\n",iso->GetMinLevel());
	}

	if(keyboard['3']) iso->SetBoundingBox(MoveDot(x,y,iso->GetMinBox()),iso->GetMaxBox());
	if(keyboard['4']) iso->SetBoundingBox(iso->GetMinBox(),MoveDot(x,y,iso->GetMaxBox()));

	if(keyboard['5']) cam.eye_distance+=mouse.dy*0.001f;
	

}

// ,    
void MouseButton ( int button, int state )
{
	mouse.Button(button,state);
	cam.MouseButton(button,state);
}

// 
void KeyButton ( int key, int state )
{
	keyboard[key]=state;
	cam.KeyButton(key,state);

	
	if(state)
	{
		
		
		if(key=='S')
			SetStereoMode((stereo_on+1)%5);
				
	}
	
}

//   
void MouseWheel( int val )
{
	static int oval = val;
	mouse.Wheel(val-oval);
	cam.MouseWheel(val-oval);
	oval = val;
}

void SetupGL()
{
	#ifdef USE_GLEW
		glewInit();
	#endif
	glClearColor(0,0,0,0);
	//glShadeModel(GL_FLAT);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
	glEnable(GL_BLEND);
	
	
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

}

int data_case=0;
void initIVP()
{
	cam.SetupProjection(45,0.3f,60.0f,0,0,width,height);


	vec3 arg;
	ivec3 size(256);
	int pixel_size=1 , type = GL_UNSIGNED_BYTE;
	std::string fname = "ct1.dat";
	
	switch(data_case)
	{
		case 1:	size.set(512,512,553);	fname = "ct2.dat"; break;
		case 2:	size.set(512,512,690);	fname = "ct3.dat"; break;
		case 3:	size.set(512,512,400);	fname = "ct4.dat"; break;
	}
	
	BYTE *arr=new BYTE[size.x*size.y*size.z*pixel_size];
	
	std::ifstream fs(fname.c_str(),std::ios::in | std::ios::binary);
	if(!fs)exit(0);

	fs.read((char*)arr,size.x*size.y*size.z*pixel_size);	//    
	fs.close();

	SetupGL();
	iso = new IsoViewer();						//        
	iso->UploadFieldData(arr,size,type);				//    GPU
	delete[] arr;

	iso->SetLevel(0.25f);
	cam.SetCenter(vec3(0.5f));
	cam.SetDistance(1.5f);

	printf("-----\n%s\n-----\n",output::out_string.c_str());	//     
	output::out_string.clear();

}

int main ( void )
{
	glfwInit ( );
	printf("'S' - change stereo mode\nUse mouse to translate/rotate/scale scene \nWhen moving mouse:\n");
	printf("	'1' - change iso-value\n");
	printf("	'3' - change BoundingBox point1\n");
	printf("	'4' - change BoundingBox point2\n");
	printf("	'5' - change distance between eyes\n");

	
	data_case = 0;


	

	int choice = 0;

	int _width, _height, mode = GLFW_WINDOW;

	//---------------------------------------------------------------------------------------------


	if ( choice == 'Y' || choice == 'y' )
	{
		GLFWvidmode vidmode;

		glfwGetDesktopMode ( &vidmode );

		width = vidmode.Width;
		
		height = vidmode.Height;

		mode = GLFW_FULLSCREEN;
	}

	//---------------------------------------------------------------------------------------------

    if( !glfwOpenWindow ( width, height, 8, 8, 8, 8, 8, 8, mode ) )
    {
        glfwTerminate ( ); exit ( 0 );
	}

	//---------------------------------------------------------------------------------------------

    glfwSwapInterval ( 0 );

	glfwSetMousePosCallback ( MouseMove );
	glfwSetMouseButtonCallback ( MouseButton );
	glfwSetKeyCallback ( KeyButton );
	glfwSetMouseWheelCallback ( MouseWheel );

	//---------------------------------------------------------------------------------------------

	

	bool running = GL_TRUE;

	int frames = 0;

	char caption [100];
	initIVP();
	double start = glfwGetTime ( );
    while ( running )
    {

        double time = glfwGetTime ( );
        if ( ( time - start ) > 1.0 || frames == 0 )
        {
            double fps = frames / ( time - start );
            sprintf_s ( caption, "%.1f FPS", fps );
            glfwSetWindowTitle ( caption );
            start = time;
            frames = 0;
        }
        frames++;

		//-----------------------------------------------------------------------------------------

        glfwGetWindowSize ( &_width, &_height );
        _height = max ( _height, 1 );
		if(_height!=height || _width!=width)
		{
			width=_width;
			height=_height;
			cam.SetupProjection(45,0.3f,60.0f,0,0,width,height);
			if(stereo_on==3)InitInterlace();

		}

		FullDraw();

        glfwSwapBuffers ( );

		//-----------------------------------------------------------------------------------------
		
		running = !glfwGetKey ( GLFW_KEY_ESC ) && glfwGetWindowParam ( GLFW_OPENED );
	}

    glfwTerminate ( ); 
	
	delete iso;

	exit ( 0 );
}